#!/bin/bash

set -e

aparse()
{
    if [ ! -z $1 ]; then
        command=$1
        shift
    else
        command=usage
        return 0
    fi

    while [[ $# > 0 ]] ; do
      case "$1" in
        --compose-flags)
            shift
              export DOCKER_COMPOSE_FLAGS=${1}
              shift
          ;;
        --server-flags)
              shift
              export DOCKER_COMPOSE_XRAY_SERVER_FLAGS=${1}
              shift
          ;;
        --persist-flags)
              shift
              export DOCKER_COMPOSE_XRAY_PERSIST_FLAGS=${1}
              shift
          ;;
        --indexer-flags)
              shift
              export DOCKER_COMPOSE_XRAY_INDEXER_FLAGS=${1}
              shift
          ;;
        --xevent-flags)
              shift
              export DOCKER_COMPOSE_XRAY_XEVENT_FLAGS=${1}
              shift
          ;;
        --analysis-flags)
              shift
              export DOCKER_COMPOSE_XRAY_ANALYSIS_FLAGS=${1}
              shift
          ;;
          *)
              ALL_ARGS="${ALL_ARGS}${1} "
              shift
          ;;
      esac
    done
}

if [ -z "$COMPOSE_PROJECT_NAME" ]; then
    COMPOSE_PROJECT_NAME="xray"
fi

# Installed version
INSTALLED_VERSION_FILE="/data/version.current"
INSTALLED_VERSION_FILE_HISTORY="/data/version.history"
INSTALLED_VERSION=''
EULA_DOC="/data/JFrog_Xray_EULA.info"
EULA_DOC_SRC="/opt/jfrog/xray/installer/JFrog_Xray_EULA.info"

if [ -f $INSTALLED_VERSION_FILE ]
then
    INSTALLED_VERSION=$(cat $INSTALLED_VERSION_FILE)
fi

# Maintainer / Installer version
MAINTAINER_VERSION_FILE="/opt/jfrog/xray/installer/version.current"
MAINTAINER_VERSION=''
if [ -f $MAINTAINER_VERSION_FILE ]
then
    MAINTAINER_VERSION=$(cat $MAINTAINER_VERSION_FILE)
fi

# Default compos env\
export DOCKER_COMPOSE_FLAGS=''
export DOCKER_COMPOSE_XRAY_SERVER_FLAGS=''
export DOCKER_COMPOSE_XRAY_PERSIST_FLAGS=''
export DOCKER_COMPOSE_XRAY_INDEXER_FLAGS=''
export DOCKER_COMPOSE_XRAY_XEVENT_FLAGS=''
export DOCKER_COMPOSE_XRAY_ANALYSIS_FLAGS=''

exitOnError()
{
    msg=$1
    echo "ERROR: ${msg}"
    exit 1
}

usage() {
    cat ./readme.txt
    exit 1
}

runDockerCommand() {
    local action=$1
    [ "$action" == "logs" ] && action="logs -f"
    docker-compose -p ${COMPOSE_PROJECT_NAME} ${action}
}

# Returned code:
#0) $1=$2
#1) $1>$2
#2) $1<$2
verComp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i
    local ver1=($1)
    local ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

isUpgradeAllowed() {

    local installedVersion=$INSTALLED_VERSION
    local maintainerVersion=$MAINTAINER_VERSION

    if [ -z $MAINTAINER_VERSION ]
    then
        exitOnError "MAINTAINER_VERSION is unknown"
    fi

    # This is a fresh installation
    if [ -z $INSTALLED_VERSION ]
    then
        return 0
    fi

    if [[ "$installedVersion" =~ ^b.* ]] && [[ "$maintainerVersion" =~ ^b.* ]]
    then
        installedVersion=($(echo $INSTALLED_VERSION | sed "s/^b//g"))
        maintainerVersion=($(echo $MAINTAINER_VERSION | sed "s/^b//g"))
    fi

    verComp $installedVersion $maintainerVersion
    if [ $? -eq 1 ]
    then
        return 1
    fi

    return 0
}

setVersions(){

    if [ ! -f ${MAINTAINER_VERSION_FILE} ]
    then
        return 1
    fi

    cat ${MAINTAINER_VERSION_FILE} > ${INSTALLED_VERSION_FILE}
    cat ${MAINTAINER_VERSION_FILE} >> ${INSTALLED_VERSION_FILE_HISTORY}

    echo "${INSTALLED_VERSION_FILE}: $(cat ${INSTALLED_VERSION_FILE})"

    return 0
}

setEULA(){
    cp ${EULA_DOC_SRC} ${EULA_DOC}
    return $?
}

createMongoUsers(){
    USERS_CREATED_FILE="/data/mongo_users.created"
    if [ ! -f "${USERS_CREATED_FILE}" ];then
      sleep 1
      cat createMongoUsers.js | docker exec -i "${COMPOSE_PROJECT_NAME}_mongodb_1" mongo  > /dev/null && \
      touch $USERS_CREATED_FILE && \
      echo -e "\nINFO: Successfully created MongoDB default users" \
      || exitOnError "Failed to create MongoDB default users"
    fi
}

##################################
############## MAIN ##############
##################################

aparse $@
case $command in
    install)
        [ -f $INSTALLED_VERSION_FILE ] && exitOnError "Xray already installed, please use upgrade"
        ( isUpgradeAllowed && \
          runDockerCommand pull && \
          setVersions && \
          setEULA \
        ) || exitOnError "Installation failed"
        echo -e "\nINFO: Xray was installed successfully, please run ./xray start"
        ;;
    upgrade)
        isUpgradeAllowed || exitOnError "Unable to upgrade to older version"

        ( runDockerCommand pull && \
          runDockerCommand stop && \
          setVersions && \
          echo -e "\nINFO: Xray was upgraded successfully, please run ./xray start" \
        ) || exitOnError "Upgrade failed"
        ;;
    start)
        runDockerCommand "up -d" && \
        createMongoUsers
        ;;
    ps|logs|stop|kill|restart)
        runDockerCommand ${command}
        ;;
    x-rm)
        runDockerCommand rm
        ;;
    version)
        if [ -f $INSTALLED_VERSION_FILE ]; then
            echo "Xray version: ${INSTALLED_VERSION}"
        else
            echo "Xray is not installed"
        fi
        ;;
    x-compose)
            xComposeCmd="docker-compose -p ${COMPOSE_PROJECT_NAME} ${ALL_ARGS}"
            echo "Advanced user: Going to execute '${xComposeCmd}'"
            ${xComposeCmd}
        ;;
    *)
        usage
        ;;
esac
